@@ -45,6 +45,7 @@
 #include <eel/eel-string-list.h>
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
+#include <eel/eel-gtk-extensions.h>
 #include <gtk/gtksignal.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
@@ -70,9 +71,9 @@
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 
-#define FACTORY_IID	     "OAFIID:nautilus_factory:bd1e1862-92d7-4391-963e-37583f0daef3"
-#define SEARCH_LIST_VIEW_IID "OAFIID:nautilus_file_manager_search_list_view:b186e381-198e-43cf-9c46-60b6bb35db0b"
-#define SHELL_IID	     "OAFIID:nautilus_shell:cd5183b2-3913-4b74-9b8e-10528b0de08d"
+#define FACTORY_IID	     "OAFIID:Nautilus_Factory"
+#define SEARCH_LIST_VIEW_IID "OAFIID:Nautilus_File_Manager_Search_List_View"
+#define SHELL_IID	     "OAFIID:Nautilus_Shell"
 
 /* Keeps track of the one and only desktop window. */
 static NautilusDesktopWindow *nautilus_application_desktop_window;
@@ -338,8 +339,9 @@ create_starthere_link_callback (gpointer data)
 	
 	/* Create default services icon on the desktop */
 	desktop_path = nautilus_get_desktop_directory ();
-	desktop_link_file = nautilus_make_path (desktop_path,
-						"starthere.desktop");
+	desktop_link_file = g_build_filename (desktop_path,
+					      "starthere.desktop",
+					      NULL);
 
 	cmd = g_strconcat ("/bin/cp ",
 			   NAUTILUS_DATADIR,
@@ -588,10 +590,13 @@ nautilus_application_create_desktop_window (NautilusApplication *application)
 	create_in_progress = TRUE;
 
 	nautilus_application_desktop_window = nautilus_desktop_window_new (application);
+	/* We realize it immediately so that the NAUTILUS_DESKTOP_WINDOW_ID
+	   property is set so gnome-settings-daemon doesn't try to set the
+	   background. And we do a gdk_flush() to be sure X gets it. */
+	gtk_widget_realize (GTK_WIDGET (nautilus_application_desktop_window));
+	gdk_flush ();
 
 	create_in_progress = FALSE;
-
-	gtk_widget_show (GTK_WIDGET (nautilus_application_desktop_window));
 }
 
 void
@@ -638,6 +643,82 @@ nautilus_window_delete_event_callback (GtkWidget *widget,
 	return TRUE;
 }				       
 
+static gboolean
+save_window_geometry_timeout (gpointer callback_data)
+{
+	NautilusWindow *window;
+	
+	window = NAUTILUS_WINDOW (callback_data);
+	
+	nautilus_window_save_geometry (window);
+
+	window->save_geometry_timeout_id = 0;
+	return FALSE;
+}
+ 
+static gboolean
+nautilus_window_configure_event_callback (GtkWidget *widget,
+						GdkEventConfigure *event,
+						gpointer callback_data)
+{
+	NautilusWindow *window;
+	char *geometry_string;
+	
+	window = NAUTILUS_WINDOW (widget);
+	
+	/* Only save the geometry if the user hasn't resized the window
+	 * for half a second. Otherwise delay the callback another half second.
+	 */
+	if (window->save_geometry_timeout_id != 0) {
+		g_source_remove (window->save_geometry_timeout_id);	
+	}
+	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (window)) && !NAUTILUS_IS_DESKTOP_WINDOW (window)) {
+		
+		geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
+	
+		/* If the last geometry is NULL the window must have just
+		 * been shown. No need to save geometry to disk since it
+		 * must be the same.
+		 */
+		if (window->last_geometry == NULL) {
+			window->last_geometry = geometry_string;
+			return FALSE;
+		}
+	
+		/* Don't save geometry if it's the same as before. */
+		if (!strcmp (window->last_geometry, geometry_string)) {
+			g_free (geometry_string);
+			return FALSE;
+		}
+
+		g_free (window->last_geometry);
+		window->last_geometry = geometry_string;
+
+		window->save_geometry_timeout_id = 
+				g_timeout_add (500, save_window_geometry_timeout, window);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+nautilus_window_unrealize_event_callback (GtkWidget *widget,
+						GdkEvent *event,
+						gpointer callback_data)
+{
+	NautilusWindow *window;
+	
+	window = NAUTILUS_WINDOW (widget);
+
+	if (window->save_geometry_timeout_id != 0) {
+		g_source_remove (window->save_geometry_timeout_id);
+		window->save_geometry_timeout_id = 0;
+		nautilus_window_save_geometry (window);
+	}
+
+	return FALSE;
+}
+
 NautilusWindow *
 nautilus_application_create_window (NautilusApplication *application)
 {
@@ -655,6 +736,12 @@ nautilus_application_create_window (NautilusApplication *application)
 	g_signal_connect_object (window, "destroy",
 				 G_CALLBACK (nautilus_application_destroyed_window), application, 0);
 
+	g_signal_connect (window, "configure_event",
+			  G_CALLBACK (nautilus_window_configure_event_callback), NULL);
+
+	g_signal_connect (window, "unrealize",
+			  G_CALLBACK (nautilus_window_unrealize_event_callback), NULL);
+
 	nautilus_application_window_list = g_list_prepend (nautilus_application_window_list, window);
 
 	/* Do not yet show the window. It will be shown later on if it can
@@ -745,14 +832,14 @@ volume_mounted_callback (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
 {
 	NautilusWindow *window;
 	char *uri;
-	
+
 	if (volume == NULL || application == NULL) {
 		return;
 	}
-	
+
 	/* Open a window to the CD if the user has set that preference. */
 	if (nautilus_volume_get_device_type (volume) == NAUTILUS_DEVICE_CDROM_DRIVE
-		&& gnome_config_get_bool ("/magicdev/Options/do_fileman_window=true")) {		
+		&& eel_gconf_get_boolean( "/apps/magicdev/do_fileman_window")) {
 		window = nautilus_application_create_window (application);
 		uri = gnome_vfs_get_uri_from_local_path (nautilus_volume_get_mount_path (volume));
 		nautilus_window_go_to (window, uri);
@@ -844,22 +931,50 @@ static gint
 save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style, gint shutdown,
 	      GnomeInteractStyle interact_style, gint fast, gpointer data)
 {
+	NautilusWindow *window;
+	GList *l;
+	static char *clone_argv[] = { "nautilus", "--no-default-window" };
+	char **restart_argv;
+	int argc;
+	int i;
+	int num_windows;
+
+	num_windows = g_list_length (nautilus_application_window_list);
+	if (num_windows > 0) {
+		argc = 1 + num_windows;
+		i = 0;
+		restart_argv = g_new (char *, argc);
+		restart_argv[i++] = g_strdup ("nautilus");
+		for (l = nautilus_application_window_list; l != NULL; l = l->next) {
+			window = NAUTILUS_WINDOW (l->data);
+			restart_argv[i++] = nautilus_window_get_location (window);
+		}
+		
+		gnome_client_set_restart_command (client, argc, restart_argv);
+
+		for (i = 0; i < argc; i++) {
+			g_free (restart_argv[i]);
+		}
+		g_free (restart_argv);
+	} else {
+		gnome_client_set_restart_command (client, 
+						  G_N_ELEMENTS (clone_argv), 
+						  clone_argv);
+	}
+	
 	return TRUE;
 }
 
 static void
 set_session_restart (GnomeClient *client, gboolean restart)
 {
-	static char *restart_argv[] = { "nautilus", "--no-default-window", 0 };
-
-	gnome_client_set_restart_command (client, 2, restart_argv);
 	gnome_client_set_priority (client, 40);
 
 	if (restart && g_getenv ("NAUTILUS_DEBUG") == NULL) {
 		/* Don't respawn in debug mode */
 		gnome_client_set_restart_style (client, GNOME_RESTART_IMMEDIATELY);
 	} else {
-		gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
+		gnome_client_set_restart_style (client, GNOME_RESTART_IF_RUNNING);
 	}
 }
 
@@ -895,6 +1010,8 @@ init_session (void)
 	update_session (client);
 }
 
+#ifdef UGLY_HACK_TO_DETECT_KDE
+
 static gboolean
 get_self_typed_prop (Window      xwindow,
                      Atom        atom,
@@ -1008,16 +1125,20 @@ look_for_kdesktop_recursive (Window xwindow)
 
 	return retval;
 }
+#endif /* UGLY_HACK_TO_DETECT_KDE */
 
 static gboolean
 is_kdesktop_present (void)
 {
+#ifdef UGLY_HACK_TO_DETECT_KDE
 	/* FIXME this is a pretty lame hack, should be replaced
 	 * eventually with e.g. a requirement that desktop managers
 	 * support a manager selection, ICCCM sec 2.8
 	 */
-
 	return look_for_kdesktop_recursive (GDK_ROOT_WINDOW ());
+#else
+	return FALSE;
+#endif
 }
 
 static void
