@@ -38,6 +38,7 @@
 #include <bonobo/bonobo-exception.h>
 
 #include <eel/eel-background.h>
+#include <eel/eel-background-style.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
@@ -85,11 +86,15 @@ struct NautilusSidebarDetails {
 	char *uri;
 	NautilusFile *file;
 	guint file_changed_connection;
-	char *default_background_color;
-	char *default_background_image;
 	int selected_index;
 	gboolean background_connected;
 	int old_width;
+
+	char *default_background_color;
+	char *default_background_image;
+	char *current_background_color;
+	char *current_background_image;
+	gboolean is_default_background;
 };
 
 /* button assignments */
@@ -118,6 +123,8 @@ static void     nautilus_sidebar_drag_data_received    (GtkWidget        *widget
 static void     nautilus_sidebar_read_theme            (NautilusSidebar  *sidebar);
 static void     nautilus_sidebar_size_allocate         (GtkWidget        *widget,
 							GtkAllocation    *allocation);
+static void     nautilus_sidebar_style_set             (GtkWidget        *widget,
+							GtkStyle         *previous_style);
 static void     nautilus_sidebar_theme_changed         (gpointer          user_data);
 static void     nautilus_sidebar_confirm_trash_changed (gpointer          user_data);
 static void     nautilus_sidebar_update_appearance     (NautilusSidebar  *sidebar);
@@ -196,6 +203,7 @@ nautilus_sidebar_class_init (GtkObjectClass *object_klass)
 	widget_class->button_press_event  = nautilus_sidebar_press_event;
 	widget_class->button_release_event  = nautilus_sidebar_release_event;
 	widget_class->size_allocate = nautilus_sidebar_size_allocate;
+	widget_class->style_set = nautilus_sidebar_style_set;
 
 	/* add the "location changed" signal */
 	signals[LOCATION_CHANGED] = g_signal_new
@@ -257,7 +265,7 @@ nautilus_sidebar_init (GtkObject *object)
 	sidebar->details->old_width = sidebar_width_auto_value;
 
 	/* load the default background from the current theme */
-	nautilus_sidebar_read_theme(sidebar);
+	nautilus_sidebar_read_theme (sidebar);
 
 	/* enable mouse tracking */
 	gtk_widget_add_events (GTK_WIDGET (sidebar), GDK_POINTER_MOTION_MASK);
@@ -349,6 +357,8 @@ nautilus_sidebar_finalize (GObject *object)
 	g_free (sidebar->details->uri);
 	g_free (sidebar->details->default_background_color);
 	g_free (sidebar->details->default_background_image);
+	g_free (sidebar->details->current_background_color);
+	g_free (sidebar->details->current_background_image);
 	g_free (sidebar->details);
 		
 	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_THEME,
@@ -529,27 +539,6 @@ sidebar_add_panel_context_menu_items (NautilusSidebar *sidebar,
 	nautilus_sidebar_for_each_panel (sidebar_for_each_sidebar_panel, &data);
 }
 
-/* check to see if the background matches the default */
-static gboolean
-nautilus_sidebar_background_is_default (NautilusSidebar *sidebar)
-{
-	char *background_color, *background_image;
-	gboolean is_default;
-	
-	background_color = nautilus_file_get_metadata (sidebar->details->file,
-						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-						       NULL);
-	background_image = nautilus_file_get_metadata (sidebar->details->file,
-						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-						       NULL);
-	
-	is_default = background_color == NULL && background_image == NULL;
-	g_free (background_color);
-	g_free (background_image);
-	
-	return is_default;
-}
-
 /* create the context menu */
 GtkWidget *
 nautilus_sidebar_create_context_menu (NautilusSidebar *sidebar)
@@ -559,25 +548,26 @@ nautilus_sidebar_create_context_menu (NautilusSidebar *sidebar)
 	gboolean has_background;
 
 	background = eel_get_widget_background (GTK_WIDGET(sidebar));
-	has_background = background && !nautilus_sidebar_background_is_default (sidebar);
-	
+	has_background = background && !sidebar->details->is_default_background;
+
 	menu = gtk_menu_new ();
 	
+	/* add the sidebar panels */
+	sidebar_add_panel_context_menu_items (sidebar, menu);
+
+	/* add a separator */
+	menu_item = gtk_menu_item_new ();
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
 	/* add the reset background item, possibly disabled */
-	menu_item = gtk_menu_item_new_with_label (_("Reset Background"));
+	menu_item = gtk_menu_item_new_with_mnemonic (_("Use _Default Background"));
  	gtk_widget_show (menu_item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
         gtk_widget_set_sensitive (menu_item, has_background);
 	g_signal_connect_object (menu_item, "activate",
 				 G_CALLBACK (reset_background_callback), sidebar, 0);
 
-	/* add a separator */
-	menu_item = gtk_menu_item_new ();
-	gtk_widget_show (menu_item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-	
-	/* add the sidebar panels */
-	sidebar_add_panel_context_menu_items (sidebar, menu);
 	return menu;
 }
 
@@ -617,20 +607,20 @@ nautilus_sidebar_read_theme (NautilusSidebar *sidebar)
 	background_color = nautilus_theme_get_theme_data ("sidebar", NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR);
 	background_image = nautilus_theme_get_theme_data ("sidebar", NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE);
 	
-	g_free(sidebar->details->default_background_color);
+	g_free (sidebar->details->default_background_color);
 	sidebar->details->default_background_color = NULL;
-	g_free(sidebar->details->default_background_image);
+	g_free (sidebar->details->default_background_image);
 	sidebar->details->default_background_image = NULL;
 			
 	if (background_color && strlen (background_color)) {
-		sidebar->details->default_background_color = g_strdup(background_color);
+		sidebar->details->default_background_color = g_strdup (background_color);
 	}
 			
 	/* set up the default background image */
 	
 	background_image = map_local_data_file (background_image);
 	if (background_image && strlen (background_image)) {
-		sidebar->details->default_background_image = g_strdup(background_image);
+		sidebar->details->default_background_image = g_strdup (background_image);
 	}
 
 	g_free (background_color);
@@ -718,7 +708,7 @@ receive_dropped_uri_list (NautilusSidebar *sidebar,
 	GtkWindow *window;
 	
 	uris = g_strsplit (selection_data->data, "\r\n", 0);
-	exactly_one = uris[0] != NULL && uris[1] == NULL;
+	exactly_one = uris[0] != NULL && (uris[1] == NULL || uris[1][0] == '\0');
 	window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar)));
 	
 	switch (hit_test (sidebar, x, y)) {
@@ -1497,9 +1487,7 @@ nautilus_sidebar_update_buttons (NautilusSidebar *sidebar)
 	 * need a framework to allow protocols to add commands buttons */
 	if (eel_istr_has_prefix (sidebar->details->uri, "trash:")) {
 		/* FIXME: We don't use spaces to pad labels! */
-		temp_button = gtk_button_new_with_mnemonic (confirm_trash_auto_value 
-							 ? _("_Empty Trash...") 
-							 : _("_Empty Trash"));
+		temp_button = gtk_button_new_with_mnemonic (_("_Empty Trash"));
 
 		gtk_box_pack_start (GTK_BOX (sidebar->details->button_box), 
 					temp_button, FALSE, FALSE, 0);
@@ -1534,6 +1522,18 @@ nautilus_sidebar_update_buttons (NautilusSidebar *sidebar)
 	}
 }
 
+static gboolean
+value_different (const char *a, const char *b)
+{
+	if (!a && !b)
+		return FALSE;
+
+	if (!a || !b)
+		return TRUE;
+
+	return strcmp (a, b);
+}
+
 static void
 nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 {
@@ -1555,17 +1555,19 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 	}
 	
 	/* Set up the background color and image from the metadata. */
+	background_color = nautilus_file_get_metadata (sidebar->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+						       NULL);
+	background_image = nautilus_file_get_metadata (sidebar->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+						       NULL);
 
-	if (nautilus_sidebar_background_is_default (sidebar)) {
+	if (background_color == NULL && background_image == NULL) {
 		background_color = g_strdup (sidebar->details->default_background_color);
 		background_image = g_strdup (sidebar->details->default_background_image);
+		sidebar->details->is_default_background = TRUE;
 	} else {
-		background_color = nautilus_file_get_metadata (sidebar->details->file,
-							       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-							       NULL);
-		background_image = nautilus_file_get_metadata (sidebar->details->file,
-							       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-							       NULL);
+		sidebar->details->is_default_background = FALSE;
 	}
 		
 	/* Block so we don't write these settings out in response to our set calls below */
@@ -1573,8 +1575,21 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 					 G_CALLBACK (background_settings_changed_callback),
 					 sidebar);
 
-	eel_background_set_image_uri (background, background_image);
-	eel_background_set_color (background, background_color);
+	if (value_different (sidebar->details->current_background_color, background_color) ||
+	    value_different (sidebar->details->current_background_image, background_image)) {
+		
+		g_free (sidebar->details->current_background_color);
+		sidebar->details->current_background_color = g_strdup (background_color);
+		g_free (sidebar->details->current_background_image);
+		sidebar->details->current_background_image = g_strdup (background_image);
+
+		eel_background_set_image_uri (background, background_image);
+		eel_background_set_color (background, background_color);
+
+		nautilus_sidebar_title_select_text_color
+			(sidebar->details->title, background,
+			 sidebar->details->is_default_background);
+	}
 
 	g_free (background_color);
 	g_free (background_image);
@@ -1693,6 +1708,22 @@ nautilus_sidebar_size_allocate (GtkWidget *widget,
 	}	
 }
 
+/* ::style_set handler for the sidebar */
+static void
+nautilus_sidebar_style_set (GtkWidget *widget, GtkStyle *previous_style)
+{
+	NautilusSidebar *sidebar;
+	GtkStyle *style;
+
+	sidebar = NAUTILUS_SIDEBAR (widget);
+
+	style = gtk_widget_get_style (widget);
+
+	/* This is slightly hackish */
+	if (!EEL_IS_BACKGROUND_STYLE (style))
+		nautilus_sidebar_theme_changed (sidebar);
+}
+
 void
 nautilus_sidebar_setup_width (NautilusSidebar *sidebar)
 {
