@@ -38,7 +38,6 @@
 #include "nautilus-switchable-navigation-bar.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-zoom-control.h"
-#include <X11/Xatom.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <eel/eel-debug.h>
@@ -180,6 +179,9 @@ nautilus_window_instance_init (NautilusWindow *window)
 	g_object_ref (G_OBJECT (window->details->tooltips));
 	gtk_object_sink (GTK_OBJECT (window->details->tooltips));
 
+	/* Set last geometry to NULL */
+	window->last_geometry = NULL;
+
 	/* Set initial window title */
 	gtk_window_set_title (GTK_WINDOW (window), _("Nautilus"));
 
@@ -215,6 +217,9 @@ ui_idle_handler (gpointer data)
 
 	window = data;
 
+	g_assert (NAUTILUS_IS_WINDOW (window));
+	g_object_ref (data);
+
 	g_assert (window->details->ui_change_depth == 0);
 
 	/* Simulate an extra freeze/thaw so that calling window_ui_freeze
@@ -239,6 +244,9 @@ ui_idle_handler (gpointer data)
 	window->details->ui_change_depth--;
 
 	window->details->ui_idle_id = 0;
+
+	g_object_unref (data);
+
 	return FALSE;
 }
 
@@ -808,7 +816,6 @@ nautilus_window_finalize (GObject *object)
 	nautilus_window_set_viewed_file (window, NULL);
 	nautilus_window_remove_bookmarks_menu_callback (window);
 	nautilus_window_remove_go_menu_callback (window);
-	nautilus_window_toolbar_remove_theme_callback (window);
 
 	if (window->details->ui_idle_id != 0) {
 		gtk_idle_remove (window->details->ui_idle_id);
@@ -859,35 +866,27 @@ nautilus_window_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
+void
 nautilus_window_save_geometry (NautilusWindow *window)
 {
 	char *geometry_string;
 
-        g_assert (NAUTILUS_IS_WINDOW (window));
-	g_assert (GTK_WIDGET_VISIBLE (window));
+	g_assert (NAUTILUS_IS_WINDOW (window));
 
-        geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
+	geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
 
 	nautilus_file_set_metadata (window->details->viewed_file,
 				    NAUTILUS_METADATA_KEY_WINDOW_GEOMETRY,
 				    NULL,
 				    geometry_string);
+				    
 	g_free (geometry_string);
 }
 
 void
 nautilus_window_close (NautilusWindow *window)
 {
-        g_return_if_fail (NAUTILUS_IS_WINDOW (window));
-
-	/* Save the window position in the directory's metadata only if
-	 * we're in every-location-in-its-own-window mode. Otherwise it
-	 * would be too apparently random when the stored positions change.
-	 */
-	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW)) {
-	        nautilus_window_save_geometry (window);
-	}
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
 	gtk_widget_destroy (GTK_WIDGET (window));
 }
@@ -1494,7 +1493,11 @@ nautilus_window_go_home (NautilusWindow *window)
 
 	nautilus_window_set_search_mode (window, FALSE);
 
+#ifdef WEB_NAVIGATION_ENABLED
 	home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+#else
+	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+#endif
 	
 	g_assert (home_uri != NULL);
 	nautilus_window_go_to (window, home_uri);
@@ -1892,6 +1895,7 @@ nautilus_window_hide_toolbar (NautilusWindow *window)
 void 
 nautilus_window_show_toolbar (NautilusWindow *window)
 {
+	nautilus_window_activate_throbber (window);
 	show_dock_item (window, TOOLBAR_PATH);
 }
 
